//
// Copyright (c) Microsoft Corporation. All rights reserved.
//
// DO NOT HAND-EDIT THIS FILE.
//
// This file was generated by setup_public_headers.py script.
// Any changes made to this file will be overwritten.

/**
 * @file att_result.h
 *
 * @brief Defines the result codes.
 *
 * @note You MUST NOT use any symbols (macros, functions, structures, enums, etc.)
 * prefixed with an underscore ('_') directly in your application code. These symbols
 * are part of the SDK's internal implementation; we do not document these symbols
 * and they are subject to change in future versions of the SDK which would break your code.
 */

#ifndef _ATT_RESULT_H
#define _ATT_RESULT_H

#include "_att_cfg.h"
#include <stdbool.h>
#include <stdint.h>

enum
{
    _att_FACILITY_GENERAL = static_cast<uint32_t>(0x1),
    _att_FACILITY_OPENSSL = static_cast<uint32_t>(0x2),
    _att_FACILITY_TSS = static_cast<uint32_t>(0x3),
    _att_FACILITY_TBS = static_cast<uint32_t>(0x4),
    _att_FACILITY_INVALID = static_cast<uint32_t>(0x7FFF),
};

enum
{
    _att_ERROR_FLAG = static_cast<uint32_t>(0x80000000),
};

constexpr auto _att_ERROR_FACILITY_MASK = static_cast<uint32_t>(0xFFFF0000);

/**
 * @brief The type represents the various success and error conditions.
 */
typedef int32_t att_result;

// att_result Bits:
//   - 31 Severity (0 - success, 1 - failure).
//   - 16..30 Facility.
//   - 0..15 Code.

#define _att_RESULT_MAKE_ERROR(facility, code) \
  (static_cast<att_result>(static_cast<uint32_t>(_att_ERROR_FLAG) | (static_cast<uint32_t>(facility) << 16U) | static_cast<uint32_t>(code & 0x0000FFFF)))

#define _att_RESULT_MAKE_SUCCESS(facility, code) \
  (static_cast<att_result>((static_cast<uint32_t>(facility) << 16U) | static_cast<uint32_t>(code & 0x0000FFFF)))

#define _att_FACILITY_MAKE_MASK(facility) \
  ((static_cast<uint32_t>(_att_ERROR_FLAG) | static_cast<uint32_t>(facility) << 16U) & _att_ERROR_FACILITY_MASK)

/**
 * @brief The type represents the various #att_result success and error conditions.
 */
enum att_result_codes
{
    // === GENERAL result codes ===

    /// Success.
    ATT_OK = _att_RESULT_MAKE_SUCCESS(_att_FACILITY_GENERAL, 0),

    /// Unknown error.
    ATT_ERROR_UNKNOWN = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 1),

    /// Internal error.
    ATT_ERROR_INTERNAL = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 2),

    /// Catastrophic failure.
    ATT_ERROR_UNEXPECTED = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 3),

    /// Input argument does not comply with the expected range of values.
    ATT_ERROR_INVALID_ARG = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 4),

    /// Invalid handle.
    ATT_ERROR_HANDLE = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 5),

    /// Requested functionality is not implemented.
    ATT_ERROR_NOT_IMPLEMENTED = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 6),

    /// Not supported.
    ATT_ERROR_NOT_SUPPORTED = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 7),

    /// Dynamic memory allocation request was not successful.
    ATT_ERROR_OUT_OF_MEMORY = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 8),

    /// General access denied error.
    ATT_ERROR_ACCESS_DENIED = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 9),

    /// A method was called at an unexpected time.
    ATT_ERROR_ILLEGAL_METHOD_CALL = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 10),

    /// The data is invalid.
    ATT_ERROR_INVALID_DATA = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 11),

    /// The resource is not in the correct state to perform the requested operation.
    ATT_ERROR_INVALID_STATE = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 12),

    /// The data area passed to a system call is too small.
    ATT_ERROR_INSUFFICIENT_BUFFER = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 13),

    /// The operation could not be completed. A retry should be performed.
    ATT_ERROR_RETRY = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 14),

    /// The access code is invalid.
    ATT_ERROR_INVALID_ACCESS = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 15),

    /// An attempt was made to perform an initialization operation when initialization has already been completed.
    ATT_ERROR_ALREADY_INITIALIZED = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 16),

    /// The client of a component requested an operation which is not valid given the state of the component instance.
    ATT_ERROR_REQUEST_OUT_OF_SEQUENCE = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 17),

    /// Cannot complete this function.
    ATT_ERROR_CAN_NOT_COMPLETE = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 18),

    /// The specified datatype is invalid.
    ATT_ERROR_INVALID_DATATYPE = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 19),

    /// The specified file was not found.
    ATT_ERROR_FILE_NOT_FOUND = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 20),

    // The IO operation failed.
    ATT_ERROR_IO = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 21),

    // The TPM device returned an error or isn't available.
    ATT_ERROR_TPM_DEVICE_ERROR = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 22),

    // The decryption failed.
    ATT_ERROR_DECRYPTION_FAILED = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 23),

    // Memory access violation.
    ATT_ERROR_ACCESS_VIOLATION = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 24),

    // The operation would cause a numeric overflow.
    ATT_ERROR_OVERFLOW = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 25),

    // Protocol version or data is invalid.
    ATT_ERROR_INVALID_PROTOCOL = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 26),

    // The requested resource is in use.
    ATT_ERROR_RESOURCE_IN_USE = _att_RESULT_MAKE_ERROR(_att_FACILITY_GENERAL, 27),
};

ATT_NODISCARD ATT_INLINE const char*
att_fmt_err_str_get(att_result result)
{
    switch (result)
    {
    case ATT_OK:
        return "Success.";
    case ATT_ERROR_UNKNOWN:
        return "Unknown error occurred.";
    case ATT_ERROR_INTERNAL:
        return "Internal error occurred.";
    case ATT_ERROR_UNEXPECTED:
        return "Unexpected error occured.";
    case ATT_ERROR_INVALID_ARG:
        return "Invalid argument or argument not supported.";
    case ATT_ERROR_HANDLE:
        return "Invalid handle provided or handle not supported.";
    case ATT_ERROR_NOT_IMPLEMENTED:
        return "Operation is not implemented.";
    case ATT_ERROR_NOT_SUPPORTED:
        return "Operation is not supported.";
    case ATT_ERROR_OUT_OF_MEMORY:
        return "Could not allocate memory for operation.";
    case ATT_ERROR_ACCESS_DENIED:
        return "Invalid access permissions.";
    case ATT_ERROR_ILLEGAL_METHOD_CALL:
        return "Illegal method call.";
    case ATT_ERROR_INVALID_DATA:
        return "Data provided is invalid or not supported.";
    case ATT_ERROR_INVALID_STATE:
        return "Operation is performed in invalid state.";
    case ATT_ERROR_INSUFFICIENT_BUFFER:
        return "Buffer provided is too small.";
    case ATT_ERROR_RETRY:
        return "Retry operation.";
    case ATT_ERROR_INVALID_ACCESS:
        return "Invalid access.";
    case ATT_ERROR_ALREADY_INITIALIZED:
        return "Value is already initialized.";
    case ATT_ERROR_REQUEST_OUT_OF_SEQUENCE:
        return "Invalid sequence of requests.";
    case ATT_ERROR_CAN_NOT_COMPLETE:
        return "Cannot complete operation.";
    case ATT_ERROR_INVALID_DATATYPE:
        return "Provided data type is invalid.";
    case ATT_ERROR_FILE_NOT_FOUND:
        return "Expected file not found.";
    case ATT_ERROR_IO:
        return "IO operation failed.";
    case ATT_ERROR_TPM_DEVICE_ERROR:
        return "TPM device returned an error or isn't available.";
    case ATT_ERROR_DECRYPTION_FAILED:
        return "Decryption failed.";
    case ATT_ERROR_ACCESS_VIOLATION:
        return "Memory access violation.";
    case ATT_ERROR_OVERFLOW:
        return "Operation would cause a numeric overflow.";
    case ATT_ERROR_INVALID_PROTOCOL:
        return "Protocol version or data is invalid.";
    case ATT_ERROR_RESOURCE_IN_USE:
        return "Requested resource is in use.";
    default:
        return "Invalid error code.";
    }
}
enum att_tbs_result_codes
{
    /// === TBS result codes ===
    // Generic TBS library error.
    ATT_ERROR_TBS_INTERNAL_ERROR = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 1),

    // Requested TBS event log is not available.
    ATT_ERROR_TBS_NO_EVENT_LOG = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 2),

    // TBS bad parameter error.
    ATT_ERROR_TBS_BAD_PARAMETER = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 3),

    // TBS invalid output pointer error.
    ATT_ERROR_TBS_INVALID_OUTPUT_POINTER = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 4),

    // TBS invalid context error.
    ATT_ERROR_TBS_INVALID_CONTEXT = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 5),

    // TBS insufficient buffer error.
    ATT_ERROR_TBS_INSUFFICIENT_BUFFER = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 6),

    // TBS IO error.
    ATT_ERROR_TBS_IO_ERROR = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 7),

    // TBS invalid context parameter error.
    ATT_ERROR_TBS_INVALID_CONTEXT_PARAM = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 8),

    // TBS service not running error.
    ATT_ERROR_TBS_SERVICE_NOT_RUNNING = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 9),

    // TBS too many TBS contexts error.
    ATT_ERROR_TBS_TOO_MANY_TBS_CONTEXTS = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 10),

    // TBS too many resources error.
    ATT_ERROR_TBS_TOO_MANY_RESOURCES = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 11),

    // TBS service start pending error.
    ATT_ERROR_TBS_SERVICE_START_PENDING = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 12),

    // TBS PPI not supported error.
    ATT_ERROR_TBS_PPI_NOT_SUPPORTED = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 13),

    // TBS command canceled error.
    ATT_ERROR_TBS_COMMAND_CANCELED = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 14),

    // TBS buffer too large error.
    ATT_ERROR_TBS_BUFFER_TOO_LARGE = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 15),

    // TBS TPM not found error.
    ATT_ERROR_TBS_TPM_NOT_FOUND = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 16),

    // TBS service disabled error.
    ATT_ERROR_TBS_SERVICE_DISABLED = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 17),

    // TBS access denied error.
    ATT_ERROR_TBS_ACCESS_DENIED = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 18),

    // TBS provisioning not allowed error.
    ATT_ERROR_TBS_PROVISIONING_NOT_ALLOWED = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 19),

    // TBS PPI function unsupported error.
    ATT_ERROR_TBS_PPI_FUNCTION_UNSUPPORTED = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 20),

    // TBS owner auth not found error.
    ATT_ERROR_TBS_OWNERAUTH_NOT_FOUND = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 21),

    // TBS provisioning incomplete error.
    ATT_ERROR_TBS_PROVISIONING_INCOMPLETE = _att_RESULT_MAKE_ERROR(_att_FACILITY_TBS, 22),
};

ATT_INLINE ATT_NODISCARD const char*
att_tbs_fmt_err_str_get(att_result result)
{
    switch (result)
    {
    case ATT_ERROR_TBS_INTERNAL_ERROR:
        return "TBS internal error occurred.";
    case ATT_ERROR_TBS_NO_EVENT_LOG:
        return "Requested TBS event log is not available.";
    case ATT_ERROR_TBS_BAD_PARAMETER:
        return "TBS bad parameter error.";
    case ATT_ERROR_TBS_INVALID_OUTPUT_POINTER:
        return "TBS invalid output pointer error.";
    case ATT_ERROR_TBS_INVALID_CONTEXT:
        return "TBS invalid context error.";
    case ATT_ERROR_TBS_INSUFFICIENT_BUFFER:
        return "TBS insufficient buffer error.";
    case ATT_ERROR_TBS_IO_ERROR:
        return "TBS IO error.";
    case ATT_ERROR_TBS_INVALID_CONTEXT_PARAM:
        return "TBS invalid context parameter error.";
    case ATT_ERROR_TBS_SERVICE_NOT_RUNNING:
        return "TBS service not running error.";
    case ATT_ERROR_TBS_TOO_MANY_TBS_CONTEXTS:
        return "TBS too many TBS contexts error.";
    case ATT_ERROR_TBS_TOO_MANY_RESOURCES:
        return "TBS too many resources error.";
    case ATT_ERROR_TBS_SERVICE_START_PENDING:
        return "TBS service start pending error.";
    case ATT_ERROR_TBS_PPI_NOT_SUPPORTED:
        return "TBS PPI not supported error.";
    case ATT_ERROR_TBS_COMMAND_CANCELED:
        return "TBS command canceled error.";
    case ATT_ERROR_TBS_BUFFER_TOO_LARGE:
        return "TBS buffer too large error.";
    case ATT_ERROR_TBS_TPM_NOT_FOUND:
        return "TBS TPM not found error.";
    case ATT_ERROR_TBS_SERVICE_DISABLED:
        return "TBS service disabled error.";
    case ATT_ERROR_TBS_ACCESS_DENIED:
        return "TBS access denied error.";
    case ATT_ERROR_TBS_PROVISIONING_NOT_ALLOWED:
        return "TBS provisioning not allowed error.";
    case ATT_ERROR_TBS_PPI_FUNCTION_UNSUPPORTED:
        return "TBS PPI function unsupported error.";
    case ATT_ERROR_TBS_OWNERAUTH_NOT_FOUND:
        return "TBS owner auth not found error.";
    case ATT_ERROR_TBS_PROVISIONING_INCOMPLETE:
        return "TBS provisioning incomplete error.";
    default:
        return "Invalid error code.";
    }
}

enum att_tss_result_codes
{
    /// === TSS result codes ===
    // Generic TSS library error.
    ATT_ERROR_TSS_ERROR = _att_RESULT_MAKE_ERROR(_att_FACILITY_TSS, 1),

    // Invalid TPM handle error. Converted from TPM2_RC_HANDLE return code.
    ATT_ERROR_TSS_INVALID_TPM_HANDLE = _att_RESULT_MAKE_ERROR(_att_FACILITY_TSS, 2),
};

ATT_NODISCARD ATT_INLINE const char*
att_tss_fmt_err_str_get(att_result result)
{
    switch (result)
    {
    case ATT_ERROR_TSS_ERROR:
        return "TSS error occurred. Check log messages for more details.";
    case ATT_ERROR_TSS_INVALID_TPM_HANDLE:
        return "Invalid TPM handle. Check log messages for more details.";
    default:
        return "Invalid error code.";
    }
}

enum att_openssl_result_codes
{
    /// === OPENSSL result codes ===
    ATT_ERROR_OPENSSL_ERROR = _att_RESULT_MAKE_ERROR(_att_FACILITY_OPENSSL, 1),

    /// Unsupported operation.
    ATT_ERROR_OPENSSL_UNSUPPORTED_ERROR = _att_RESULT_MAKE_ERROR(_att_FACILITY_OPENSSL, 2),
};

ATT_NODISCARD ATT_INLINE const char*
att_openssl_fmt_err_str_get(att_result result)
{
    switch (result)
    {
        case ATT_ERROR_OPENSSL_ERROR:
            return "OpenSSL error occurred. Check log messages for more details.";
        case ATT_ERROR_OPENSSL_UNSUPPORTED_ERROR:
            return "Unsupported OpenSSL operation. Check log messages for more details.";
        default:
            return "Invalid error code.";
    }
}

/**
 * @brief Checks whether the \p result provided indicates a failure.
 *
 * @param[in] result Result value to check for failure.
 *
 * @return `true` if the operation that returned this \p result failed, otherwise return `false`.
 */
ATT_NODISCARD ATT_INLINE bool att_result_failed(att_result result)
{
    return (static_cast<uint32_t>(result) & static_cast<uint32_t>(_att_ERROR_FLAG)) != 0;
}

/**
 * @brief Checks whether the \p result provided indicates a success.
 *
 * @param[in] result Result value to check for success.
 *
 * @return `true` if the operation that returned this \p result was successful, otherwise return
 * `false`.
 */
ATT_NODISCARD ATT_INLINE bool att_result_succeeded(att_result result)
{
    return !att_result_failed(result);
}

/**
 * @brief Returns a string representation of the \p result provided.
 * 
 * @param[in] result Result value to decode to string.
 * @return const char* string representation of the \p result provided.
 */
ATT_NODISCARD ATT_INLINE const char*
att_general_fmt_err_str_get(att_result result)
{
    if (att_result_succeeded(result) || (result & _att_ERROR_FACILITY_MASK) == _att_FACILITY_MAKE_MASK(_att_FACILITY_GENERAL))
    {
        return att_fmt_err_str_get(result);
    }
    else if ((result & _att_ERROR_FACILITY_MASK) == _att_FACILITY_MAKE_MASK(_att_FACILITY_OPENSSL))
    {
        return att_openssl_fmt_err_str_get(result);
    }
    else if ((result & _att_ERROR_FACILITY_MASK) ==  _att_FACILITY_MAKE_MASK(_att_FACILITY_TSS))
    {
        return att_tss_fmt_err_str_get(result);
    }
    else if ((result & _att_ERROR_FACILITY_MASK) ==  _att_FACILITY_MAKE_MASK(_att_FACILITY_TBS))
    {
        return att_tbs_fmt_err_str_get(result);
    }

    return "Invalid error code.";
}

/**
 * @brief Returns a string representation of the \p result provided.
 * 
 * @param[in] result Result value to decode to string.
 * @return const char* string representation of the \p result provided.
 */
ATT_NODISCARD ATT_INLINE const char* att_result_decode(att_result result)
{
    return att_general_fmt_err_str_get(result);
}

#endif // _ATT_RESULT_H